"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HeliSabotageSystem = exports.HeliSabotageSystemRepairTag = void 0;
const SystemStatus_1 = require("./SystemStatus");
const events_1 = require("../events");
exports.HeliSabotageSystemRepairTag = {
    FixBit: 0x10,
    DeactiveBit: 0x20,
    ActiveBit: 0x40,
    DamageBit: 0x80
};
/**
 * Represents a system responsible for handling communication sabotages on The Skeld and Polus.
 *
 * See {@link HeliSabotageSystemEvents} for events to listen to.
 */
class HeliSabotageSystem extends SystemStatus_1.SystemStatus {
    constructor(ship, systemType, data) {
        var _a, _b, _c, _d;
        super(ship, systemType, data);
        this._syncTimer = 1;
        (_a = this.countdown) !== null && _a !== void 0 ? _a : (this.countdown = 10000);
        (_b = this.resetTimer) !== null && _b !== void 0 ? _b : (this.resetTimer = -1);
        (_c = this.activeConsoles) !== null && _c !== void 0 ? _c : (this.activeConsoles = new Map);
        (_d = this.completedConsoles) !== null && _d !== void 0 ? _d : (this.completedConsoles = new Set([0, 1]));
    }
    /**
     * Whether or not communications is sabotaged.
     */
    get sabotaged() {
        return this.completedConsoles.size < 2;
    }
    patch(data) {
        this.countdown = data.countdown;
        this.resetTimer = data.resetTimer;
        this.activeConsoles = data.activeConsoles;
        this.completedConsoles = data.completedConsoles;
    }
    /* eslint-disable-next-line @typescript-eslint/no-unused-vars */
    Deserialize(reader, spawn) {
        this.countdown = reader.float();
        this.resetTimer = reader.float();
        const numActive = reader.packed();
        const beforeActive = this.activeConsoles;
        this.activeConsoles = new Map;
        for (let i = 0; i < numActive; i++) {
            const playerId = reader.uint8();
            const consoleId = reader.uint8();
            const player = this.room.getPlayerByPlayerId(playerId);
            if (player) {
                this._openConsole(consoleId, player, undefined);
            }
        }
        for (const [playerId, consoleId] of beforeActive) {
            const player = this.ship.room.getPlayerByPlayerId(playerId);
            if (player) {
                this._closeConsole(consoleId, player, undefined);
            }
        }
        const numCompleted = reader.packed();
        const beforeCompleted = this.completedConsoles;
        this.completedConsoles = new Set;
        for (let i = 0; i < numCompleted; i++) {
            const consoleId = reader.uint8();
            this._completeConsole(consoleId, undefined, undefined);
        }
        if (beforeCompleted.size === 2 && numCompleted === 0) {
            this.emitSync(new events_1.SystemSabotageEvent(this.room, this, undefined, undefined));
        }
        if (beforeCompleted.size < 2 && numCompleted === 2) {
            this.emitSync(new events_1.SystemRepairEvent(this.room, this, undefined, undefined));
        }
    }
    /* eslint-disable-next-line @typescript-eslint/no-unused-vars */
    Serialize(writer, spawn) {
        writer.float(this.countdown);
        writer.float(this.resetTimer);
        writer.packed(this.activeConsoles.size);
        for (const [playerId, consoleId] of this.activeConsoles) {
            writer.uint8(playerId);
            writer.uint8(consoleId);
        }
        writer.packed(this.completedConsoles.size);
        for (const consoleId of this.completedConsoles) {
            writer.uint8(consoleId);
        }
    }
    async HandleSabotage(player, rpc) {
        const oldCountdown = 10000;
        const oldTimer = this.resetTimer;
        const oldActive = this.activeConsoles;
        const oldCompleted = this.completedConsoles;
        this.countdown = 90;
        this.resetTimer = -1;
        this.activeConsoles = new Map;
        this.completedConsoles = new Set;
        this.dirty = true;
        const ev = await this.emit(new events_1.SystemSabotageEvent(this.room, this, rpc, player));
        if (ev.reverted) {
            this.countdown = oldCountdown;
            this.resetTimer = oldTimer;
            this.activeConsoles = oldActive;
            this.completedConsoles = oldCompleted;
        }
    }
    async _resetConsoles() {
        const oldCompleted = this.completedConsoles;
        this.completedConsoles = new Set;
        this.dirty = true;
        const ev = await this.emit(new events_1.HeliSabotageConsolesResetEvent(this.room, this));
        if (ev.reverted) {
            this.completedConsoles = oldCompleted;
        }
    }
    async _openConsole(consoleId, player, rpc) {
        if (player.playerId === undefined)
            return;
        if (this.activeConsoles.has(player.playerId))
            return;
        this.activeConsoles.set(player.playerId, consoleId);
        this.dirty = true;
        const ev = await this.emit(new events_1.HeliSabotageConsoleOpenEvent(this.room, this, rpc, player, consoleId));
        if (ev.reverted) {
            this.activeConsoles.delete(player.playerId);
            return;
        }
        if (ev.alteredConsoleId !== consoleId) {
            this.activeConsoles.set(player.playerId, ev.alteredConsoleId);
        }
    }
    async openConsoleAs(consoleId, player) {
        await this._openConsole(consoleId, player, undefined);
    }
    /**
     * Mark the console as being used by your player.
     * @param consoleId The ID of the console to mark as being used.
     */
    async openConsole(consoleId) {
        if (!this.room.myPlayer)
            return;
        if (this.room.hostIsMe) {
            await this.openConsoleAs(consoleId, this.room.myPlayer);
        }
        else {
            this._sendRepair(0x40 | consoleId);
        }
    }
    async _closeConsole(consoleId, player, rpc) {
        if (player.playerId === undefined)
            return;
        if (!this.activeConsoles.delete(player.playerId))
            return;
        this.dirty = true;
        const ev = await this.emit(new events_1.HeliSabotageConsoleCloseEvent(this.room, this, rpc, player, consoleId));
        if (ev.reverted) {
            this.activeConsoles.delete(player.playerId);
        }
    }
    async closeConsoleAs(consoleId, player) {
        await this._closeConsole(consoleId, player, undefined);
    }
    /**
     * Mark the console as no longer being used by your player.
     * @param consoleId The ID of the console to mark as not being used.
     */
    async closeConsole(consoleId) {
        if (!this.room.myPlayer)
            return;
        if (this.room.hostIsMe) {
            await this.closeConsoleAs(consoleId, this.room.myPlayer);
        }
        else {
            await this._sendRepair(0x20 | consoleId);
        }
    }
    async _completeConsole(consoleId, player, rpc) {
        if (!this.completedConsoles.has(consoleId)) {
            this.completedConsoles.add(consoleId);
            this.dirty = true;
            const ev = await this.emit(new events_1.HeliSabotageConsoleCompleteEvent(this.room, this, rpc, player, consoleId));
            if (ev.reverted) {
                this.completedConsoles.delete(consoleId);
            }
        }
    }
    /**
     * Mark a console as completed.
     * @param consoleId The ID of the console to mark as completed.
     */
    async completeConsole(consoleId) {
        if (this.room.hostIsMe) {
            await this._completeConsole(consoleId, this.room.myPlayer, undefined);
        }
        else {
            await this._sendRepair(0x10 | consoleId);
        }
    }
    async _repair(player, rpc) {
        const completedBefore = this.completedConsoles;
        const timerBefore = this.resetTimer;
        const countdownBefore = this.countdown;
        this.completedConsoles = new Set([0, 1]);
        this.resetTimer = -1;
        this.countdown = 10000;
        this.dirty = true;
        const ev = await this.emit(new events_1.SystemRepairEvent(this.room, this, rpc, player));
        if (ev.reverted) {
            this.resetTimer = timerBefore;
            this.completedConsoles = completedBefore;
            this.countdown = countdownBefore;
        }
    }
    async repair() {
        if (this.room.hostIsMe) {
            await this._repair(this.room.myPlayer, undefined);
        }
        else {
            await this.completeConsole(0);
            await this.completeConsole(1);
        }
    }
    async HandleRepair(player, amount, rpc) {
        const consoleId = amount & 0xf;
        const repairOperation = amount & 0xf0;
        switch (repairOperation) {
            case exports.HeliSabotageSystemRepairTag.DeactiveBit:
                if (!player)
                    return;
                this._closeConsole(consoleId, player, rpc);
                break;
            case exports.HeliSabotageSystemRepairTag.ActiveBit:
                if (!player)
                    return;
                this._openConsole(consoleId, player, rpc);
                break;
            case exports.HeliSabotageSystemRepairTag.DamageBit:
                this.HandleSabotage(player, rpc);
                break;
            case exports.HeliSabotageSystemRepairTag.FixBit:
                this._completeConsole(consoleId, player, rpc);
                break;
        }
    }
    Detoriorate(delta) {
        if (!this.sabotaged)
            return;
        this.resetTimer -= delta;
        if (this.resetTimer < 0) {
            this.resetTimer = 10;
            this.countdown -= delta;
            this._resetConsoles();
            this._syncTimer -= delta;
            if (this._syncTimer <= 0) {
                this._syncTimer = 1;
                this.dirty = true;
            }
        }
    }
}
exports.HeliSabotageSystem = HeliSabotageSystem;
//# sourceMappingURL=HeliSabotageSystem.js.map